@@ -56,6 +56,8 @@ struct _DzlSignalGroup
   GPtrArray  *handlers;
   GType       target_type;
   gsize       block_count;
+
+  guint       has_bound_at_least_once : 1;
 };
 
 struct _DzlSignalGroupClass
@@ -124,13 +126,12 @@ dzl_signal_group__target_weak_notify (gpointer  data,
                                       GObject  *where_object_was)
 {
   DzlSignalGroup *self = data;
-  gsize i;
 
   g_assert (DZL_IS_SIGNAL_GROUP (self));
   g_assert (where_object_was != NULL);
   g_assert (self->target == where_object_was);
 
-  for (i = 0; i < self->handlers->len; i++)
+  for (guint i = 0; i < self->handlers->len; i++)
     {
       SignalHandler *handler;
 
@@ -182,7 +183,10 @@ dzl_signal_group_bind_handler (DzlSignalGroup *self,
   g_assert (handler != NULL);
   g_assert (handler->signal_id != 0);
   g_assert (handler->closure != NULL);
-  g_assert (handler->handler_id == 0);
+
+  /* Possibly a re-entrancy issue */
+  if (handler->handler_id != 0)
+    return;
 
   handler->handler_id = g_signal_connect_closure_by_id (self->target,
                                                         handler->signal_id,
@@ -200,8 +204,6 @@ static void
 dzl_signal_group_bind (DzlSignalGroup *self,
                        GObject        *target)
 {
-  gsize i;
-
   g_assert (DZL_IS_SIGNAL_GROUP (self));
   g_assert (self->target == NULL);
   g_assert (!target || G_IS_OBJECT (target));
@@ -209,14 +211,16 @@ dzl_signal_group_bind (DzlSignalGroup *self,
   if (target == NULL)
     return;
 
+  self->has_bound_at_least_once = TRUE;
+
+  g_object_ref (target);
+
   self->target = target;
   g_object_weak_ref (self->target,
                      dzl_signal_group__target_weak_notify,
                      self);
 
-  g_object_ref (target);
-
-  for (i = 0; i < self->handlers->len; i++)
+  for (guint i = 0; i < self->handlers->len; i++)
     {
       SignalHandler *handler;
 
@@ -225,30 +229,39 @@ dzl_signal_group_bind (DzlSignalGroup *self,
     }
 
   g_signal_emit (self, signals [BIND], 0, target);
+
   g_object_unref (target);
 }
 
 static void
 dzl_signal_group_unbind (DzlSignalGroup *self)
 {
-  GObject *target;
-  gsize i;
+  g_autoptr(GObject) target = NULL;
 
   g_return_if_fail (DZL_IS_SIGNAL_GROUP (self));
 
-  if (self->target == NULL)
-    return;
+  if (NULL != (target = g_steal_pointer (&self->target)))
+    {
+      /*
+       * If we have a valid target, we want to hold a reference to it
+       * while we do the disconnections, just to ensure it is not
+       * finalized out from underneath us.
+       */
+      g_object_ref (target);
 
-  target = self->target;
-  self->target = NULL;
+      /*
+       * Let go of our weak reference now that we have a full reference
+       * for the life of this function.
+       */
+      g_object_weak_unref (target,
+                           dzl_signal_group__target_weak_notify,
+                           self);
+    }
 
-  g_object_weak_unref (target,
-                       dzl_signal_group__target_weak_notify,
-                       self);
-
-  for (i = 0; i < self->handlers->len; i++)
+  for (guint i = 0; i < self->handlers->len; i++)
     {
       SignalHandler *handler;
+      gulong handler_id;
 
       handler = g_ptr_array_index (self->handlers, i);
 
@@ -256,15 +269,17 @@ dzl_signal_group_unbind (DzlSignalGroup *self)
       g_assert (handler->signal_id != 0);
       g_assert (handler->closure != NULL);
 
-      if (handler->handler_id != 0)
-        {
-          gulong handler_id;
+      handler_id = handler->handler_id;
+      handler->handler_id = 0;
 
-          handler_id = handler->handler_id;
-          handler->handler_id = 0;
+      /*
+       * If @target is NULL, we lost a race to cleanup the weak
+       * instance and the signal connections have already been
+       * finalized and therefore nothing to do.
+       */
 
-          g_signal_handler_disconnect (target, handler_id);
-        }
+      if (target != NULL && handler_id != 0)
+        g_signal_handler_disconnect (target, handler_id);
     }
 
   g_signal_emit (self, signals [UNBIND], 0);
@@ -407,8 +422,12 @@ dzl_signal_group_set_target (DzlSignalGroup *self,
   if (!dzl_signal_group_check_target_type (self, target))
     return;
 
-  dzl_signal_group_unbind (self);
+  /* Only emit unbind if we've ever called bind */
+  if (self->has_bound_at_least_once)
+    dzl_signal_group_unbind (self);
+
   dzl_signal_group_bind (self, target);
+
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_TARGET]);
 }
 
@@ -448,7 +467,9 @@ dzl_signal_group_dispose (GObject *object)
 {
   DzlSignalGroup *self = (DzlSignalGroup *)object;
 
-  dzl_signal_group_unbind (self);
+  if (self->has_bound_at_least_once)
+    dzl_signal_group_unbind (self);
+
   g_clear_pointer (&self->handlers, g_ptr_array_unref);
 
   G_OBJECT_CLASS (dzl_signal_group_parent_class)->dispose (object);
