@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at your
+ * the Free Software Foundation; either version 2.1 of the License, or (at your
  * option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
@@ -1073,8 +1073,8 @@ dzl_tab_set_action_target_value (GtkActionable *actionable,
   if (priv->action_target_value != variant)
     {
       g_clear_pointer (&priv->action_target_value, g_variant_unref);
-      if (variant)
-        priv->action_target_value = g_variant_ref (variant);
+      if (variant != NULL)
+        priv->action_target_value = g_variant_ref_sink (variant);
       g_object_notify (G_OBJECT (self), "action-target");
     }
 }
